from datetime import timedelta, datetime
from functools import lru_cache, wraps
from logging import Logger
from typing import List

logger = Logger("pymultirole")


def comma_separated_to_list(commasep: str, sep: str = ",") -> List[str]:
    commasep_aslist: List[str] = []
    if commasep:
        commasep_aslist = [x.strip() for x in commasep.split(sep)]
    return commasep_aslist


def timed_lru_cache(seconds: int, maxsize: int = None):
    def wrapper_cache(func):
        # print('I will use lru_cache')
        func = lru_cache(maxsize=maxsize)(func)
        # print('I\'m setting func.lifetime')
        func.lifetime = timedelta(seconds=seconds)
        # print('I\'m setting func.expiration')
        func.expiration = datetime.utcnow() + func.lifetime

        @wraps(func)
        def wrapped_func(*args, **kwargs):
            # print('Check func expiration')
            # print(f'datetime.utcnow(): {datetime.utcnow()}, func.expiration: {func.expiration}')
            if datetime.utcnow() >= func.expiration:
                logger.info("func.expiration lru_cache lifetime expired")
                func.cache_clear()
                func.expiration = datetime.utcnow() + func.lifetime

            return func(*args, **kwargs)

        return wrapped_func

    return wrapper_cache



from typing import Type

import pytest
from pydantic import BaseModel, Field
from starlette.responses import Response, JSONResponse

from pymultirole_plugins.v1.formatter import FormatterBase, FormatterParameters
from pymultirole_plugins.v1.schema import Document


def test_formatter():
    with pytest.raises(TypeError) as err:
        parser = FormatterBase()
        assert parser is None
    assert (
        "Can't instantiate abstract class FormatterBase with abstract methods format"
        in str(err.value)
    )


def test_default_options():
    options = FormatterParameters()
    assert options is not None


class DummyParameters(FormatterParameters):
    foo: str = Field("foo", description="Foo")
    bar: int = Field(0, description="Bar")


class DummyFormatter(FormatterBase):
    """Dummy formatter."""

    def format(self, document: Document, parameters: FormatterParameters) -> Response:
        parameters: DummyParameters = parameters
        output = {"text": document.text, "foo": parameters.foo, "bar": parameters.bar}
        return JSONResponse(content=output)

    @classmethod
    def get_model(cls) -> Type[BaseModel]:
        return DummyParameters


def test_dummy():
    formatter = DummyFormatter()
    options = DummyParameters()
    resp: Response = formatter.format(
        Document(text="This is a test document", metadata=options.dict()), options
    )
    assert resp.status_code == 200
    assert resp.media_type == "application/json"
